require 'forwardable'
require 'cgi'

module RailsTemplatedUrls
  class Builder
    extend Forwardable

    def initialize(name, params, url_helpers, routes)
      @name = name
      @params = params
      @url_helpers = url_helpers
      @routes = routes

      check_route
    end

    def_delegators :@routes, :named_routes

    def url
      build(:url)
    end

    def path
      build(:path)
    end

    protected

    def check_route
      return if named_routes.key?(@name)
      fail UnknownRouteError, "Unknown route #{@name.inspect}"
    end

    def required_parts
      named_routes[@name].required_parts
    end

    def required_parts_expressions
      required_parts.map do |key|
        { key => format('{%s}', key) }
      end
    end

    def params_expression
      return if @params.empty?
      format('{?%s}', @params.join(','))
    end

    def build(strategy)
      url = @url_helpers.public_send("#{@name}_#{strategy}",
                                     *required_parts_expressions)

      "#{CGI.unescape(url)}#{params_expression}"
    end
  end
end
